@@ -1,14 +1,17 @@
 // fontprogress.cpp
 //
-// (C) 2001 Stefan Kebekus
+// (C) 2001--2004 Stefan Kebekus
 // Distributed under the GPL
 
+#include <config.h>
+
 #include "fontprogress.h"
 
 #include <kdebug.h>
 #include <klocale.h>
+#include <kprocio.h>
 #include <kprogress.h>
-#include <kpushbutton.h>
+#include <qapplication.h>
 #include <qframe.h>
 #include <qlabel.h>
 #include <qlayout.h>
@@ -22,12 +25,12 @@
  *  Constructs a fontProgressDialog which is a child of 'parent', with the 
  *  name 'name' and widget flags set to 'f' 
  */
-fontProgressDialog::fontProgressDialog( QString helpIndex, QString label, QString abortTip, QString whatsThis, QString ttip, QWidget* parent, const QString &name, bool progressbar )
+fontProgressDialog::fontProgressDialog(const QString& helpIndex, const QString& label, const QString& abortTip, const QString& whatsThis, const QString& ttip, QWidget* parent, const QString& name, bool progressbar)
   : KDialogBase( parent, "Font Generation Progress Dialog", true, name, Cancel, Cancel, true )
 {
   setCursor( QCursor( 3 ) );
 
-  setButtonCancelText( i18n("Abort"), abortTip );
+  setButtonCancel(KGuiItem(i18n("Abort"), "stop", abortTip));
 
   if (helpIndex.isEmpty() == false) {
     setHelp(helpIndex, "kdvi");
@@ -57,6 +60,8 @@ fontProgressDialog::fontProgressDialog( QString helpIndex, QString label, QStrin
   QToolTip::add( TextLabel2, ttip );
 
   progress = 0;
+  procIO = 0;
+  qApp->connect(this, SIGNAL(finished()), this, SLOT(killProcIO()));
 }
 
 
@@ -69,7 +74,8 @@ fontProgressDialog::~fontProgressDialog()
     // no need to delete child widgets, Qt does it all for us
 }
 
-void fontProgressDialog::increaseNumSteps( const QString explanation)
+
+void fontProgressDialog::increaseNumSteps(const QString& explanation)
 {
   if (ProgressBar1 != 0)
     ProgressBar1->setProgress(progress++);
@@ -77,13 +83,9 @@ void fontProgressDialog::increaseNumSteps( const QString explanation)
 }
 
 
-void fontProgressDialog::hideDialog(void)
-{
-  hide();
-}
-
-void fontProgressDialog::setTotalSteps(int steps)
+void fontProgressDialog::setTotalSteps(int steps, KProcIO *proc)
 {
+  procIO = proc;
   if (ProgressBar1 != 0) {
     ProgressBar1->setTotalSteps(steps);
     ProgressBar1->setProgress(0);
@@ -91,9 +93,12 @@ void fontProgressDialog::setTotalSteps(int steps)
   progress = 0;
 }
 
-void fontProgressDialog::show(void)
+
+void fontProgressDialog::killProcIO()
 {
-  KDialogBase::show();
+  if (!procIO.isNull())
+    procIO->kill();
 }
 
+
 #include "fontprogress.moc"
