@@ -1,5 +1,5 @@
 /*
-** $Id: gif-info.c,v 1.5 2003/03/07 22:09:24 waba Exp $
+** $Id: gif-info.c 532487 2006-04-22 06:04:41Z mpyne $
 **
 ** Minimal GIF parser, for use in extracting and setting metadata.
 ** Modified for standalone & KDE calling by Bryce Nesbitt
@@ -50,7 +50,7 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **
 */
 
@@ -105,7 +105,7 @@ static long int pos;
 
 static char     true[] = "True";
 static char     false[] = "False";
-static char     id[] = "$Id: gif-info.c,v 1.5 2003/03/07 22:09:24 waba Exp $";
+static char     id[] = "$Id: gif-info.c 532487 2006-04-22 06:04:41Z mpyne $";
 
 /*****************************************************************************/
 
@@ -285,9 +285,8 @@ FILE    *dest;
                 gct_size=2<<(lsd[4]&0x7);
                 pos=ftell(src);
                 (void)fread((void *)gct,gct_size,3,src);
-        }
-        if (output) {
-                (void)fwrite((void *)gct,gct_size,3,dest);
+                if (output)
+                        (void)fwrite((void *)gct,gct_size,3,dest);
         }
 
         do {
@@ -479,6 +478,7 @@ FILE *      outfile;
     }
 
   if ((infile = fopen(original_filename, READ_BINARY)) == NULL) {
+    fclose( outfile );
     fprintf(stderr, "can't open gif image '%s'\n", original_filename);
     return(ERROR_NOT_A_JPEG);
     }
