@@ -40,6 +40,7 @@
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kfiledialog.h>
+#include <kiconloader.h>
 #include <kimagefilepreview.h>
 #include <kimageio.h>
 #include <kio/netaccess.h>
@@ -48,6 +49,7 @@
 #include <kprinter.h>
 #include <kstdaccel.h>
 #include <kstdaction.h>
+#include <kscan.h>
 
 #include <kpdefs.h>
 #include <kpdocument.h>
@@ -60,6 +62,9 @@
 // private
 void kpMainWindow::setupFileMenuActions ()
 {
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "kpMainWindow::setupFileMenuActions()" << endl;
+#endif
     KActionCollection *ac = actionCollection ();
 
     m_actionNew = KStdAction::openNew (this, SLOT (slotNew ()), ac);
@@ -67,6 +72,9 @@ void kpMainWindow::setupFileMenuActions ()
 
     m_actionOpenRecent = KStdAction::openRecent (this, SLOT (slotOpenRecent (const KURL &)), ac);
     m_actionOpenRecent->loadEntries (kapp->config ());
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "\trecent URLs=" << m_actionOpenRecent->items () << endl;
+#endif
 
     m_actionSave = KStdAction::save (this, SLOT (slotSave ()), ac);
     m_actionSaveAs = KStdAction::saveAs (this, SLOT (slotSaveAs ()), ac);
@@ -74,6 +82,9 @@ void kpMainWindow::setupFileMenuActions ()
     m_actionExport = new KAction (i18n ("E&xport..."), 0,
         this, SLOT (slotExport ()), ac, "file_export");
 
+    m_actionScan = new KAction (i18n ("Scan..."), SmallIcon ("scanner"), 0,
+        this, SLOT (slotScan ()), ac, "file_scan");
+
     //m_actionRevert = KStdAction::revert (this, SLOT (slotRevert ()), ac);
     m_actionReload = new KAction (i18n ("Reloa&d"), KStdAccel::reload (),
         this, SLOT (slotReload ()), ac, "file_revert");
@@ -92,6 +103,8 @@ void kpMainWindow::setupFileMenuActions ()
     m_actionClose = KStdAction::close (this, SLOT (slotClose ()), ac);
     m_actionQuit = KStdAction::quit (this, SLOT (slotQuit ()), ac);
 
+    m_scanDialog = 0;
+
     enableFileMenuDocumentActions (false);
 }
 
@@ -123,12 +136,6 @@ void kpMainWindow::enableFileMenuDocumentActions (bool enable)
 }
 
 
-// private
-bool kpMainWindow::shouldOpenInNewWindow () const
-{
-    return (m_document && !m_document->isEmpty ());
-}
-
 // private
 void kpMainWindow::addRecentURL (const KURL &url)
 {
@@ -182,22 +189,25 @@ void kpMainWindow::addRecentURL (const KURL &url)
         #endif
 
             if (mw != this)
-                mw->setRecentURLs (m_actionOpenRecent->items ());
+            {
+                // WARNING: Do not use KRecentFilesAction::setItems()
+                //          - it does not work since only its superclass,
+                //          KSelectAction, implements setItems() and can't
+                //          update KRecentFilesAction's URL list.
+
+                // Avoid URL memory leak in KRecentFilesAction::loadEntries().
+                mw->m_actionOpenRecent->clearURLList ();
+
+                mw->m_actionOpenRecent->loadEntries (cfg);
+            #if DEBUG_KP_MAIN_WINDOW
+                kdDebug () << "\t\t\tcheck recent URLs="
+                           << mw->m_actionOpenRecent->items () << endl;
+            #endif
+            }
         }
     }
 }
 
-// private
-void kpMainWindow::setRecentURLs (const QStringList &items)
-{
-#if DEBUG_KP_MAIN_WINDOW
-    kdDebug () << "kpMainWindow(" << name () << ")::setRecentURLs()" << endl;
-    kdDebug () << "\titems=" << items << endl;
-#endif
-    m_actionOpenRecent->setItems (items);
-}
-
-
 
 // private slot
 void kpMainWindow::slotNew ()
@@ -260,39 +270,69 @@ void kpMainWindow::saveDefaultDocSize (const QSize &size)
 
 
 // private
-bool kpMainWindow::open (const KURL &url, bool newDocSameNameIfNotExist)
+bool kpMainWindow::shouldOpenInNewWindow () const
 {
-    QSize docSize = defaultDocSize ();
-
-    // create doc
-    kpDocument *newDoc = new kpDocument (docSize.width (), docSize.height (), this);
-    if (newDoc->open (url, newDocSameNameIfNotExist))
-    {
-        if (newDoc->isFromURL (false/*don't bother checking exists*/))
-            addRecentURL (url);
-    }
-    else
-    {
-        delete newDoc;
-        return false;
-    }
+    return (m_document && !m_document->isEmpty ());
+}
 
+// private
+void kpMainWindow::setDocumentChoosingWindow (kpDocument *doc)
+{
     // need new window?
     if (shouldOpenInNewWindow ())
     {
         // send doc to new window
-        kpMainWindow *win = new kpMainWindow (newDoc);
+        kpMainWindow *win = new kpMainWindow (doc);
         win->show ();
     }
     else
     {
         // set up views, doc signals
-        setDocument (newDoc);
+        setDocument (doc);
+    }
+}
+
+
+// private
+kpDocument *kpMainWindow::openInternal (const KURL &url,
+        const QSize &fallbackDocSize,
+        bool newDocSameNameIfNotExist)
+{
+    // create doc
+    kpDocument *newDoc = new kpDocument (fallbackDocSize.width (),
+                                         fallbackDocSize.height (),
+                                         this);
+    if (!newDoc->open (url, newDocSameNameIfNotExist))
+    {
+        delete newDoc;
+        return 0;
     }
 
-    return true;
+    // Send document to current or new window.
+    setDocumentChoosingWindow (newDoc);
+
+    return newDoc;
 }
 
+// private
+bool kpMainWindow::open (const KURL &url, bool newDocSameNameIfNotExist)
+{
+    kpDocument *newDoc = openInternal (url,
+                                       defaultDocSize (),
+                                       newDocSameNameIfNotExist);
+    if (newDoc)
+    {
+        if (newDoc->isFromURL (false/*don't bother checking exists*/))
+            addRecentURL (url);
+        return true;
+    }
+    else
+    {
+        return false;
+    }
+}
+
+
 // private
 KURL::List kpMainWindow::askForOpenURLs (const QString &caption, const QString &startURL,
                                          bool allowMultipleURLs)
@@ -345,12 +385,161 @@ void kpMainWindow::slotOpenRecent (const KURL &url)
 {
 #if DEBUG_KP_MAIN_WINDOW
     kdDebug () << "kpMainWindow::slotOpenRecent(" << url << ")" << endl;
+    kdDebug () << "\titems=" << m_actionOpenRecent->items () << endl;
 #endif
 
     if (toolHasBegunShape ())
         tool ()->endShapeInternal ();
 
     open (url);
+
+    // If the open is successful, addRecentURL() would have bubbled up the
+    // URL in the File / Open Recent action.  As a side effect, the URL is
+    // deselected.
+    //
+    // If the open fails, we should deselect the URL:
+    //
+    // 1. for consistency
+    //
+    // 2. because it has not been opened.
+    //
+    m_actionOpenRecent->setCurrentItem (-1);
+}
+
+
+// private slot
+void kpMainWindow::slotScan ()
+{
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "kpMainWindow::slotScan() scanDialog=" << m_scanDialog << endl;
+#endif
+
+    if (toolHasBegunShape ())
+        tool ()->endShapeInternal ();
+
+
+    if (!m_scanDialog)
+    {
+        // Create scan dialog by looking for plugin.
+        // [takes about 500ms on 350Mhz]
+        m_scanDialog = KScanDialog::getScanDialog (this, "scandialog", true/*modal*/);
+
+        // No scanning support (kdegraphics/libkscan) installed?
+        // [Remove $KDEDIR/share/servicetypes/kscan.desktop and
+        //         $KDEDIR/share/services/scanservice.desktop to simulate this]
+        if (!m_scanDialog)
+        {
+        #if DEBUG_KP_MAIN_WINDOW
+            kdDebug () << "\tcould not create scan dialog" << endl;
+        #endif
+            // Instead, we could try to create the scan dialog in the ctor
+            // and just disable the action in the first place, removing
+            // the need for this dialog.
+            //
+            // But this increases startup time and is a bit risky e.g. if
+            // the scan support hangs, KolourPaint would not be able to be
+            // started at all.
+            //
+            // Also, disabling the action is bad because the scan support
+            // can be installed while KolourPaint is still running.
+            KMessageBox::sorry (this,
+                         i18n ("Scanning support is not installed."),
+                         i18n ("No Scanning Support"));
+            return;
+        }
+
+    #if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\tcreated scanDialog=" << m_scanDialog << endl;
+    #endif
+        connect (m_scanDialog, SIGNAL (finalImage (const QImage &, int)),
+                 SLOT (slotScanned (const QImage &, int)));
+    }
+
+
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "\tcalling setup" << endl;
+#endif
+    // Bring up dialog to select scan device.
+    if (m_scanDialog->setup ())
+    {
+    #if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\t\tOK - showing dialog" << endl;
+    #endif
+        // Called only if scanner configured/available.
+        //
+        // In reality, this seems to be called even if you press "Cancel" in
+        // the KScanDialog::setup() dialog!
+        m_scanDialog->show ();
+    }
+    else
+    {
+        // Have never seen this code path execute even if "Cancel" is pressed.
+    #if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\t\tFAIL" << endl;
+    #endif
+    }
+}
+
+// private slot
+void kpMainWindow::slotScanned (const QImage &image, int)
+{
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "kpMainWindow::slotScanned() image.rect=" << image.rect () << endl;
+#endif
+
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "\thiding dialog" << endl;
+#endif
+    // (KScanDialog does not close itself after a scan is made)
+    //
+    // Close the dialog, first thing:
+    //
+    // 1. This means that any dialogs we bring up won't be nested on top.
+    //
+    // 2. We don't want to return from this method but forget to close
+    //    the dialog.  So do it before anything else.
+    m_scanDialog->hide ();
+
+    // (just in case there's some drawing between slotScan() exiting and
+    //  us being called)
+    if (toolHasBegunShape ())
+        tool ()->endShapeInternal ();
+
+
+    // TODO: Maybe this code should be moved into kpdocument.cpp -
+    //       since it resembles the responsibilities of kpDocument::open().
+
+    // Convert QImage to kpDocument's image format, gathering meta info
+    // from QImage.
+    kpDocumentSaveOptions saveOptions;
+    kpDocumentMetaInfo metaInfo;
+    const QPixmap pixmap = kpDocument::convertToPixmapAsLosslessAsPossible (
+        image,
+        kpMainWindow::pasteWarnAboutLossInfo (),
+        &saveOptions,
+        &metaInfo);
+
+    if (pixmap.isNull ())
+    {
+    #if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\tcould not convert to pixmap" << endl;
+    #endif
+        KMessageBox::sorry (this,
+                            i18n ("Cannot scan - out of graphics memory."),
+                            i18n ("Cannot Scan"));
+        return;
+    }
+
+
+    // Create document from image and meta info.
+    kpDocument *doc = new kpDocument (pixmap.width (), pixmap.height (), this);
+    doc->setPixmap (pixmap);
+    doc->setSaveOptions (saveOptions);
+    doc->setMetaInfo (metaInfo);
+
+
+    // Send document to current or new window.
+    setDocumentChoosingWindow (doc);
 }
 
 
@@ -361,7 +550,7 @@ bool kpMainWindow::save (bool localOnly)
         KImageIO::mimeTypes (KImageIO::Writing)
             .findIndex (m_document->saveOptions ()->mimeType ()) < 0 ||
         // SYNC: kpDocument::getPixmapFromFile() can't determine quality
-        //       from file
+        //       from file so it has been set initially to an invalid value.
         (m_document->saveOptions ()->mimeTypeHasConfigurableQuality () &&
             m_document->saveOptions ()->qualityIsInvalid ()) ||
         (localOnly && !m_document->url ().isLocalFile ()))
@@ -867,7 +1056,7 @@ void kpMainWindow::sendPixmapToPrinter (KPrinter *printer,
         // according to the screen's DPI.
         // TODO: I think we should use the image's DPI.  Technically
         //       possible?
-        //       
+        //
         //       So no matter what computer you draw text on, you get
         //       the same pixels.
         //
@@ -908,7 +1097,7 @@ void kpMainWindow::sendPixmapToPrinter (KPrinter *printer,
                << endl;
 #endif
 
-    
+
     double dpiX = pixmapDotsPerMeterX / InchesPerMeter;
     double dpiY = pixmapDotsPerMeterY / InchesPerMeter;
 #if DEBUG_KP_MAIN_WINDOW
@@ -1045,6 +1234,9 @@ void kpMainWindow::slotPrintPreview ()
 // private slot
 void kpMainWindow::slotMail ()
 {
+    if (toolHasBegunShape ())
+        tool ()->endShapeInternal ();
+
     if (m_document->url ().isEmpty ()/*no name*/ ||
         !m_document->isFromURL () ||
         m_document->isModified ()/*needs to be saved*/)
