@@ -45,6 +45,8 @@ class AboutWidget;
 class KURL;
 class KURLComboBox;
 
+class KuickFile;
+
 class DelayedRepeatEvent
 {
 public:
@@ -104,8 +106,8 @@ private slots:
     void 		slotPrint();
     void 		slotConfigApplied();
     void 		slotConfigClosed();
-    void 		messageCantLoadImage( const QString& );
-    void         	showImage(const KFileItem *, bool newWindow = false,
+    void 		messageCantLoadImage( const KuickFile * file, const QString& message);
+    bool         	showImage(const KFileItem *, bool newWindow = false,
                                   bool fullscreen = false, bool moveToTopLeft = true );
     void 		showFileItem( ImageWindow *, const KFileItem * );
     void		slotHighlighted( const KFileItem * );
@@ -146,6 +148,7 @@ private:
     void 		saveSettings();
     bool 		haveBrowser() const;
     void 		delayedRepeatEvent( ImageWindow *, QKeyEvent * );
+    void		abortDelayedEvent();
     void                deleteAllViewers();
     void                redirectDeleteAndTrashActions(KActionCollection *coll);
 
