@@ -68,7 +68,9 @@ KMultiPage::KMultiPage(QWidget *parentWidget, const char *widgetName, QObject *p
   // Create MarkList
   _markList = new MarkList(sideBar, "marklist");
   sideBar->addItem(_markList, QIconSet(SmallIcon("thumbnail")), i18n("Thumbnails"));
-  sideBar->setCurrentItem(_markList);
+
+  // Restore state of the sidebar
+  sideBar->setCurrentItem(sideBar->item(KVSPrefs::sideBarItem()));
 
   splitterWidget->setResizeMode(sideBar, QSplitter::KeepSize);
 
@@ -101,14 +103,28 @@ KMultiPage::KMultiPage(QWidget *parentWidget, const char *widgetName, QObject *p
 
 KMultiPage::~KMultiPage()
 {
-  KVSPrefs::setGuiLayout(splitterWidget->sizes());
-  KVSPrefs::writeConfig();
+  writeSettings();
+
   if (timer_id != -1)
     killTimer(timer_id);
 
   delete pageCache;
 }
 
+void KMultiPage::readSettings()
+{
+}
+
+void KMultiPage::writeSettings()
+{
+  // Save TOC layout
+  tableOfContents->writeSettings();
+
+  KVSPrefs::setGuiLayout(splitterWidget->sizes());
+  // Save state of the sidebar
+  KVSPrefs::setSideBarItem(sideBar->indexOf(sideBar->currentItem()));
+  KVSPrefs::writeConfig();
+}
 
 QString KMultiPage::name_of_current_file()
 {
@@ -151,7 +167,7 @@ void KMultiPage::slotSave()
 
   if (QFile(fileName).exists()) {
     int r = KMessageBox::warningContinueCancel (0, i18n("The file %1\nexists. Shall I overwrite that file?").arg(fileName),
-				       i18n("Overwrite File"), i18n("OverWrite"));
+				       i18n("Overwrite File"), i18n("Overwrite"));
     if (r == KMessageBox::Cancel)
       return;
   }
