@@ -228,6 +228,7 @@ KScanDevice::~KScanDevice()
   if( storeOptions )  delete (storeOptions );
   kdDebug(29000) << "Calling sane_exit to finish sane!" << endl;
   sane_exit();
+  delete d;
 }
 
 
@@ -307,7 +308,7 @@ QString KScanDevice::getScannerName(const QCString& name) const
   {
      scanner = scannerDevices[ scanner_name ];
   }
-  else
+  else if ( ! name.isEmpty() )
   {
      scanner = scannerDevices[ name ];
      ret = QString::null;
@@ -704,6 +705,9 @@ bool KScanDevice::savePreviewImage( const QImage &image )
 
 KScanStat KScanDevice::acquirePreview( bool forceGray, int dpi )
 {
+   if( ! scanner_handle )
+      return KSCAN_ERR_NO_DEVICE;
+
    double min, max, q;
 
    (void) forceGray;
@@ -901,6 +905,9 @@ void KScanDevice::prepareScan( void )
  **/
 KScanStat KScanDevice::acquire( const QString& filename )
 {
+    if( ! scanner_handle )
+       return KSCAN_ERR_NO_DEVICE;
+
     KScanOption *so = 0;
 
     if( filename.isEmpty() )
@@ -1194,6 +1201,7 @@ void KScanDevice::slScanFinished( KScanStat status )
 	// put the resolution also into the image itself
 	img->setDotsPerMeterX(static_cast<int>(d->currScanResolutionX / 0.0254 + 0.5));
 	img->setDotsPerMeterY(static_cast<int>(d->currScanResolutionY / 0.0254 + 0.5));
+	kdDebug(29000) << "resolutionX:" << d->currScanResolutionX << " resolutionY:" << d->currScanResolutionY << endl;
 
 	if( scanningPreview )
 	{
